class BigNum {

	int [] digits;
	//Idea: each element of the array is a digit
	//To simplify implementation: each digit is a normal decimal digit 0-9
	//Most important digit is in position 0

	public BigNum(int val){ //Initialize a BigNum to value val
		int len = 0;
		int tmp = val;
		while(tmp>0) { len++; tmp = tmp/10; }
		if(val==0) len = 1;
		digits = new int [len];
		for(int i=digits.length-1;i>=0;i--){
			digits[i] = val%10;
			val = val/10;
		}

	}

	public BigNum(){
		this(0);
	}

	private BigNum trim(){ //Return a shorter BigNum by dropping leading 0s
		int i=0;
		while(i<digits.length && digits[i]==0) i++;
		if(i==0) return this;
		if(i==digits.length) return new BigNum(0);
		BigNum b = new BigNum();
		b.digits = new int [digits.length-i];
		for(int j=0; j<b.digits.length; i++,j++)
			b.digits[j] = digits[i];
		return b;
	}


	public String toString(){
		String s = new String();
		int i=0;
		while(i<digits.length && digits[i]==0 ) i++; //Drop initial 0s
		for(;i<digits.length;i++)
			s = s+(Integer.toString(digits[i]));
		if("".equals(s)) s="0"; //Note Yoda syntax
		return s;
	}

	public boolean equals(BigNum b){ 
		BigNum a = this.trim();
		b = b.trim();
		if(a.digits.length != b.digits.length) return false;
		for(int i=0;i<a.digits.length;i++){
			if(a.digits[i]!=b.digits[i]) return false;
		}
		return true;
	}

	public int hashCode(){//Technically required because we are re-defining equals. Students may skip this...
		int res=0;
		for(int i=0;i<digits.length;i++)
			res+=digits[i];
		return res;
	} 

	public BigNum add(BigNum b){ 
		if(b.digits.length > digits.length) return b.add(this); //assume that this has more digits

		int i,j;
		int carry = 0;
		BigNum res = new BigNum(0);
		res.digits = new int[digits.length+1];
		for(i=digits.length-1, j=b.digits.length-1; i>=0 && j>=0; i--,j--){
			res.digits[i+1] = digits[i] + b.digits[j] + carry ;
			carry = res.digits[i+1] /10;
			res.digits[i+1] = res.digits[i+1]%10;
		}		
		//digits of b are now done
		while(i>=0){
			res.digits[i+1] = digits[i] + carry ;
			carry = res.digits[i+1] /10;
			res.digits[i+1] = res.digits[i+1]%10;
			i--;
		}

		if(carry==0) return res;//.trim();
		//Overflow!
		res.digits[0] = carry;
		return res;

	}
	
	private BigNum multiplyBy10(){
		BigNum res = new BigNum(0);
		res.digits = new int [digits.length+1];
		for(int i=0;i<digits.length;i++){
			res.digits[i] = digits[i];
		}
		return res;
	}

	private BigNum multiplyByDigit(int d){
		BigNum res = new BigNum();
		res.digits = new int[digits.length+1];
		int carry = 0;
		for(int i=digits.length-1;i>=0;i--){
			res.digits[i+1] = d*digits[i] + carry;
			carry = res.digits[i+1]/10;
			res.digits[i+1] = res.digits[i+1]%10;
		}
		if(carry == 0) return res.trim();
		//Overflow!
		res.digits[0]=carry;
		return res;
	}

	public BigNum multiply(BigNum b){
		b = b.trim(); //Drop leading zeros
		BigNum res = new BigNum();
		BigNum tmp;
		for(int i=0; i<b.digits.length; i++){
			res = res.multiplyBy10();
			tmp = this.multiplyByDigit(b.digits[i]);
			res = res.add(tmp);
		}
		return res;
	}

	public boolean isBigger(BigNum b){
		BigNum a = this.trim();
		b = b.trim();
		if(a.digits.length > b.digits.length) return true;
		if(a.digits.length < b.digits.length) return false;
		if(a.equals(b)) return false;
		for(int i=0; i<a.digits.length; i++)
			if(a.digits[i]>b.digits[i]) return true;
			else if(b.digits[i]>a.digits[i]) return false;
		//!!!
		System.out.println("This should not happen!");
		return false;
	}

	public BigNum subtract(BigNum b){
		if(this.equals(b)) return new BigNum(0);
		if(!this.isBigger(b)) return b.subtract(this); //Always return absolute value of difference
		BigNum a = this.trim();
		b = b.trim();
		int carry = 0,i,j;
		BigNum res = new BigNum();
		res.digits = new int[a.digits.length];
		for(i=a.digits.length-1, j=b.digits.length-1; i>=0 && j>=0; i--, j--){
			res.digits[i] = a.digits[i] - b.digits[j] - carry;
			if(res.digits[i] < 0){ res.digits[i]+=10; carry = 1;} else carry = 0;
		}
		//Done with b
		while(i>=0){
			res.digits[i] = a.digits[i]  - carry;
			if(res.digits[i] < 0){ res.digits[i]+=10; carry = 1;} else carry = 0;
			i--;
		}
		return res.trim();
	}

	public BigNum div(BigNum b){
		return fulldiv(b)[0];
	}

	public BigNum mod(BigNum b){
		return fulldiv(b)[1];
	}

	private BigNum[] fulldiv(BigNum d){
		BigNum q = new BigNum();
		BigNum r ;
		BigNum n = this;
		BigNum tmp;
		int i;
		while(!d.isBigger(n)){
			if(d.equals(n)){
				q = q.add(new BigNum(1));
				break;
			}
			//find first digit
			for(tmp = new BigNum(1); n.isBigger(tmp.multiply(d).multiply(new BigNum(10))); tmp = tmp.multiply(new BigNum(10)));
			for(i=1; i<10; i++){
				if(!n.isBigger(tmp.multiply(d).multiply(new BigNum(i)))) break;
			}
			i--;
			tmp = tmp.multiply(new BigNum(i));

			q = q.add(tmp);
			n = n.subtract(tmp.multiply(d));
		}
		r = n.trim();
		BigNum[] res = new BigNum [2];
		res[0] = q;
		res[1] = r;
		return res;
	}
}
